/*
 * Copyright 2024 Arne Vajh�j.
 * 
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *    http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License. 
 */

package dk.vajhoej.vms.tig.client;

import java.io.FileWriter;
import java.io.IOException;
import java.io.PrintWriter;

import dk.vajhoej.vms.tig.Arg;
import dk.vajhoej.vms.tig.Arg.DataType;
import dk.vajhoej.vms.tig.Arg.Usage;
import dk.vajhoej.vms.tig.Method;
import dk.vajhoej.vms.tig.shared.Util;

public class PyClientGen implements ClientGen {
	private PrintWriter pw;
	@Override
	public String fileName(String imgnam) {
		return className(imgnam) + ".py";
	}
	@Override
	public String className(String imgnam) {
		return Util.capitalize(imgnam) + "Client";
	}
	@Override
	public String typeName(Arg.DataType typ) {
		switch(typ) {
			case Byte:
				return "int8";
			case Word:
				return "int16";
			case LongWord:
				return "int32";
			case QuadWord:
				return "int64";
			case F_Float:
			case S_Float:
				return "float";
			case D_Float:
			case G_Float:
			case T_Float:
				return "double";
			case FixedCharacterString:
			case VariableCharacterString:
			case NullTermCharacterString:
				return "string";
			case Block:
				return "block";
			default:
				throw new RuntimeException("Unsupported type: " + typ);
		}
	}
	@Override
	public void open(String fnm) throws IOException {
		pw = new PrintWriter(new FileWriter(fnm));
	}
	@Override
	public void genClassHeader(String ns, String clznam, String ooctx) {
		pw.printf("# GENERATED CODE - edits may be overwritten\n");
		pw.printf("import tig\n");
		if(ooctx != null && !Character.isDigit(ooctx.charAt(0))) {
			pw.printf("from %s import *\n", Util.capitalize(ooctx));
		}
		pw.printf("\n");
		pw.printf("class %s(object):\n", clznam);
	}
	@Override
	public void genCtor(String clznam, int port, String ooctx) {
		pw.printf("    def __init__(self, host):\n", clznam);
		pw.printf("        self.host = host\n");
		pw.printf("        self.port = %d\n", port);
		if(ooctx != null) {
			if(!Character.isDigit(ooctx.charAt(0))) {
				pw.printf("        self._ooctx = %s()\n", Util.capitalize(ooctx));
				pw.printf("    def oocontext(self):\n");
				pw.printf("        return self._ooctx\n");
			} else {
				pw.printf("        self._ooctx = bytearray(%d)\n", Integer.parseInt(ooctx));
			}
		}
	}
	@Override
	public void genMethodHeader(String methnam, Method.ArgList args) {
		pw.printf("    def %s(self", methnam);
		for(Arg arg : args.getArgs()) {
			if(arg.getUse() != Usage.Out) {
				pw.printf(", %s", arg.getName());
			}
		}
		pw.printf("):\n");
		pw.printf("        self.con = tig.Connection(self.host, self.port)\n");
		pw.printf("        self.con.prep()\n");
		pw.printf("        self.con.put_string(\"%s\")\n", methnam);
	}
	@Override
	public void genMethodBody(String ooctx, Method.ArgList args) {
		if(ooctx != null) {
			if(!Character.isDigit(ooctx.charAt(0))) {
				pw.printf("        self.con.put_block(self._ooctx)\n");
			} else {
				pw.printf("        self.con.put_bytes(self._ooctx)\n");
			}
		}
		for(Arg arg : args.getArgs()) {
			if(arg.getUse() != Usage.Out || arg.getType() == DataType.Block) {
		        pw.printf("        self.con.put_%s(%s)\n", typeName(arg.getType()), arg.getName());
			}
		}
		pw.printf("        self.con.send()\n");
		pw.printf("        self.con.recv()\n");
		pw.printf("        retval = self.con.get_int32()\n");
		if(ooctx != null) {
			if(!Character.isDigit(ooctx.charAt(0))) {
				pw.printf("        self.con.get_block(self._ooctx)\n");
			} else {
				pw.printf("        self._ooctx = self.con.get_bytes()\n");
			}
		}
		for(Arg arg : args.getArgs()) {
			if(arg.getUse() != Usage.In) {
				if(arg.getType() != DataType.Block) {
					pw.printf("        %s = self.con.get_%s()\n", arg.getName(), typeName(arg.getType()));
				} else {
					//pw.printf("        %s = %s()\n", arg.getName(), arg.getImpl());
					pw.printf("        self.con.get_%s(%s)\n", typeName(arg.getType()), arg.getName());
				}
			}
		}
	}
	@Override
	public void genMethodFooter(Method.ArgList args) {
		pw.printf("        self.con.close()\n");
		pw.printf("        return retval");
		for(Arg arg : args.getArgs()) {
			if(arg.getUse() != Usage.In) {
				pw.printf(", %s", arg.getName());
			}
		}
		pw.printf("\n");
	}
	@Override
	public void genClassFooter() {
	}
	@Override
	public void close() {
		pw.close();
	}
}
